import json

from . import settings_manupulation
from . import paths

# get settings data
settings = settings_manupulation.settings

# todo
todo = []


def add(id=None, name=None, duration=None):
    '''
    add a new task
    '''

    global todo  
    
    # use default values if arguments are not provided
    if(not(name)):
        name = settings['default']['name']

    if(not(duration)):
        duration = settings['default']['duration']

    # create task
    task = {
        "name": name,
        "duration": int(duration),
        }

    # if task ID is provided, insert new task there
    if(type(id)==int):
        todo .insert(id, task)

    # else, append
    else:
        todo .append(task)

    # write task data
    write()

    return(True)

def remove(id=None, every=False):
    '''
    remove task
    '''

    global todo 

    # remove everything
    if(every):
        todo = []
        write()
        return(False)

    # if todo list is empty then exit
    if(not(todo)):
        print("Todo list is empty")
        return(False)

    # if ID is provided then remove that task
    if(type(id)==int):
        todo.pop(id)

    # else, remove the last task
    else:
        todo.pop()

    # write task data
    write()

    return(True)

def write():
    '''
    write task data
    '''

    with open(paths.todo_path, 'w') as data:
        json.dump(todo, data)

    return(True)

def read():
    '''
    read task data
    '''

    global todo  
    try:
        with open(paths.todo_path, 'r') as data:
            todo = json.load(data)
    except FileNotFoundError:
        print('Todo file does not exist. Creating a new one.')
        write()

    return(True)

read()



import json

from . import theme_manupulation
from . import data_manupulation
from . import paths
from . import settings_manupulation
from . import time_formatting
from . import todo

# get settings data
settings = settings_manupulation.settings

# get tasks data
tasks = data_manupulation.tasks

def create(id=None, name=None, duration=None, skip=False, done=False, start_at=None):
    '''
    create a new task
    '''

    global tasks
    
    # use default values if arguments are not provided
    if(not(name)):
        name = settings['default']['name']

    if(not(duration)):
        duration = settings['default']['duration']

    # create task
    task = {
        "name": name,
        "duration": int(duration),
        'skip': skip,
        "done": done,
        }

    # find the appropriate id if start time is provided
    if(type(start_at)==int):
        task['start_at'] = start_at
        tasks.insert(get_next(start_at), task)

    # else, use the id if provided
    elif(type(id)==int):
        tasks.insert(id, task)

    # else, append
    else:
        tasks.append(task)

    # write task data
    data_manupulation.write()
    return(True)

def task_remove(id=None, every=False):
    '''
    remove task
    '''

    global tasks

    # remove every Tasks
    if(every):
        for _ in range(len(tasks)):
            task_remove(0)
        return(True)

    # if ID is provided then remove that task
    if(type(id)==int):
        tasks.pop(id)

    # else, remove the last task
    else:
        tasks.pop()

    # write task data
    data_manupulation.write()
    
    return(True)

def task_modify(id=None, new_id=None, name=None, duration=None, start_at=None, every=False):
    '''
    modify task
    '''

    global tasks

    # modify every tasks
    if(every):
        for i in range(len(tasks)):
            task_modify(id=i, new_id=new_id, name=name, duration=duration, start_at=start_at, every=False)
        return(True)

    # if no ID is passed then pick the last task
    if(type(id) != int):
        id=len(tasks)-1

    # modify name
    if(name):
        tasks[id]['name'] = name

    # modify duration
    if(duration):
        tasks[id]['duration'] = duration

    # modify task start time
    if(type(start_at)==int):
        # remove start time
        if(start_at == -1):
            tasks[id].pop('start_at', None)

        # change start time
        else:
            new_id = get_next(start_at) # get the new ID now that start time is changed
            tasks[id]['start_at'] = start_at

    # modify ID
    if(type(new_id)==int):
        task = tasks[id]
        tasks.pop(id)
        tasks.insert(new_id, task)
    data_manupulation.write()

def display(tasks, id=None):
    '''
    display tasks
    '''

    # load theme
    theme = theme_manupulation.load()

    # if ID is provided, print that task
    if(type(id)==int):
        if(id<len(tasks)):
            print(tasks[id])
            return(True)
        else:
            raise ValueError(f"No Task with the {theme['highlight']['id']}ID {id}{theme['escape']} exists.")

    # time passed's initial value is the time user set for when their day start
    time = settings['time_start']

    def get_attr_lengths():
        '''
        # get max lenghts of each attributes for padding
        '''

        lengths = []
        lengths.append(max(len(str(len(tasks))), 2))
        lengths.append(max(len(time_formatting.to_time(sum(tasks[i]['duration'] for i in range(len(tasks)))+time)), 4))

        for attr in ['name', 'duration', 'skip', 'done']:
            length = 8
            if(len(tasks)):
                length = max(max(len(str(tasks[i][attr])) for i in range(len(tasks))), 8)
            lengths.append(length)
        return(lengths)

    lengths = get_attr_lengths()

    def print_header(name, l):
        '''
        # header printing functino
        '''

        print(theme['highlight']['header'], end='')
        print(f"{name:{l}}", end='')
        print(theme['escape'], end=' ')

    def print_attr(attr, l):
        '''
        # attribute printing function
        '''

        print(f"{str(attr):{l}}", end=' ')

    # print headers
    for attr, i in zip(['ID', 'Time', 'Name', 'Duration', 'Skip', 'Done'], lengths):
        print_header(attr, i)
    print()

    # new task that is neither done nor marked to skip
    next_undone = get_first({'done':False, 'skip': False})

    # for code reuse
    keys = ['skip', 'done']

    for i in range(len(tasks)):
        # if task has start time
        if('start_at' in tasks[i]):
            if(type(tasks[i]['start_at'])==int):
                time = tasks[i]['start_at']

        # print highlighting
        if(tasks[i]['done']): # done task
            print(theme['highlight']['done'], end='')
        elif(tasks[i]['skip']): # skipped task
            print(theme['highlight']['skip'], end='')
        else:
            if(i == next_undone): # next task to do
                print(theme['highlight']['next']['col'], end='')
            else: # undone task
                if(i%2==0):
                    print(theme['highlight']['undone']['even'], end='')
                else:
                    print(theme['highlight']['undone']['odd'], end='')

        # print ID
        print_attr(i, lengths[0])

        # print time
        print_attr(time_formatting.to_time(time), lengths[1])

        # print name
        print_attr(tasks[i]['name'], lengths[2])

        # print duration
        print_attr(time_formatting.to_time(tasks[i]['duration']), lengths[3])

        # print other attributes
        for j, key in enumerate(keys):
            value = tasks[i][key]
            print_attr(value, lengths[j+4])

        # increase time
        time += tasks[i]['duration']

        if(i == next_undone and theme['highlight']['next']['pointer']):
            print(theme['highlight']['next']['pointer'], end='') # print pointer
        print(theme['escape'])

    # the time last task ends
    print(f"{theme['highlight']['skip']}-> {time_formatting.to_time(time)}{theme['escape']}") 


    # display todo list
    todo.read()
    if(todo.todo):
        # print headers
        print_header('ID', lengths[0]+lengths[1])
        print_header('Todo', lengths[2])
        print_header('Duration', lengths[3])
        print()

        # print attributes
        for i, task in enumerate(todo.todo):
            print(theme['highlight']['undone']['even'], end='')
            print_attr(i, lengths[0]+lengths[1])
            print_attr(task['name'], lengths[2])
            print_attr(time_formatting.to_time(task['duration']), lengths[3])
            print(theme['escape'])
        return(True)

def display_today(id=None):
    '''
    display today
    '''

    display(tasks=tasks, id=id)

def display_yesterday(id=None):
    '''
    display yesterday
    '''

    try:
        with open(paths.yesterday_path, 'r') as data:
            yesterday = json.load(data)
            display(tasks=yesterday, id=id)

    except FileNotFoundError:
        print("There is no Data file for Yesterday's tasks. Yesterday's data file is only generated when newday function is called.")



def task_do(id):
    '''
    mark a task as done
    '''

    # get first undone task if ID is not provided
    if(type(id) != int):
        id = get_first({'done': False, 'skip':False})
        if(type(id) != int):
            print("All Tasks are already done")
            return(False)

    # if task is already done
    if(tasks[id]['done']):
        print(f"Task {id}: {tasks[id]['name']} was already done.")

    # else, mark task as done
    else:
        tasks[id]['done'] = True
        print(f"\033[91mTask {id}\033[0m: \33[33m{tasks[id]['name']}\033[0m done.")

        # write data
        data_manupulation.write()

def task_undo(id=None):
    '''
    mark a task as undone
    '''

    # get first done task if ID is not provided
    if(type(id) != int):
        id = get_first({'done': True, 'skip':False}, -1)
        if(type(id) != int):
            print("All Tasks are already undone")
            return(False)

    # if task is already marked undone
    if(not(tasks[id]['done'])):
        print(f"Task {id}: {tasks[id]['name']} was not marked done.")

    # else, mark undone the task
    else:
        tasks[id]['done'] = False
        print(f"\033[91mTask {id}\033[0m: \33[33m{tasks[id]['name']}\033[0m is marked undone.")

        # write data
        data_manupulation.write()

def task_do_all():
    '''
    mark all tasks as done
    '''

    for id in range(len(tasks)):
        tasks[id]['done'] = True

    # write data
    data_manupulation.write()

def task_undo_all():
    '''
    mark all tasks as undone
    '''

    for id in range(len(tasks)):
        tasks[id]['done'] = False

    # write data
    data_manupulation.write()

def task_skip(id=None, every=False):
    '''
    mark skip of a task
    '''
    global tasks

    # skip every Tasks
    if(every):
        for task in tasks:
            task['skip'] = True
        data_manupulation.write()
        return(True)

    # get the next undone task
    if(type(id) != int):
        id = get_first({'done': False, 'skip': False})
        if(type(id) != int):
            print("All Tasks are already done")
            return(False)

    # skip task
    tasks[id]['skip'] = True

    # write data
    data_manupulation.write()

    return(True)
    
def task_unskip(id=None, every=False):
    '''
    unmark skip of a task
    '''
    global tasks

    # unskip every Tasks
    if(every):
        for task in tasks:
            task['skip'] = False 
        data_manupulation.write()
        return(True)

    # get the last skipped task
    if(type(id) != int):
        id = get_first({'skip': True}, -1)
        if(type(id) != int):
            print("No tasks are skipped")
            return(False)

    # toggle skip of task
    tasks[id]['skip'] = False

    # write data
    data_manupulation.write()

    return(True)

def get_first(d: dict, step=1):
    '''
    get first task that matches given key:value
    '''

    def check(i, key):
        return(tasks[i][key] == d[key])
    for i in range(len(tasks))[::step]:
        matches = [check(i, key) for key in d]
        if(all(matches)):
            return(i)
    return(False)
 
def get_next(given_time):
    '''
    get the ID where a task will fit based on it's time
    '''

    # time when the day starts
    time = settings['time_start']

    # get to the point when time is larger than given time, return that ID
    for i, task in enumerate(tasks):
        if 'start_at' in task:
            time = task['start_at']

        time += task['duration']

        if(time > given_time):
            return(i)

    # if we never reach that point, return length
    return(len(tasks))

def save_todo(id, a_id=None, every=False):
    '''
    save a task to todo list
    '''

    # save every tasks
    if(every):
        for i in range(len(tasks)):
            save_todo(i, a_id=a_id, every=False)
    
    # use the last task if no ID is provided
    if(not(type(id)==int)):
        id = len(tasks)-1

    # add task to todo
    todo.add(a_id, tasks[id]['name'], tasks[id]['duration'])

    return(True)


def load_todo(id, a_id, every=False):
    '''
    load a task from todo list
    '''

    # load every tasks
    if(every):
        for i in range(len(todo.todo)):
            load_todo(i, a_id=a_id, every=False)

    # if todo list is empty then exist
    if(not(todo.todo)):
        print("Todo list is empty")
        return(False)


    # use the first task in todo list if no ID is provided
    if(not(type(id)==int)):
        id = 0

    # add task to data
    create(a_id, todo.todo[id]['name'], todo.todo[id]['duration'])

    # remove task from todo list
    todo.remove(id)

def copy_todo_yesterday():
    '''
    copy all undone tasks from yesterday's data to todo list
    '''

    try:
        with open(paths.yesterday_path, 'r') as data:
            yesterday = json.load(data)
            for task in yesterday:
                if(not(task['done'])):
                    todo.add(None, task['name'], task['duration'])

    except FileNotFoundError:
        print("There is no Data file for Yesterday's tasks. Yesterday's data file is only generated when newday function is called.")





import re

def to_time(m):
    '''
    format minutes to H:m
    '''

    hour = m//60%24
    minute = m%60
    t = f"{hour:>2}:{minute:0>2}"
    return(t)

def to_min(duration):
    '''
    turn string to minutes
    '''

    # INT (minutes)
    if(duration.isnumeric()):
        return(int(duration))

    # hour:minutes format
    if(':' in duration):
        s  = duration.split(':')
        return(int(s[0])*60 + int(s[1]))

    # 3h / 30m / etc.
    m = re.match("^(\d+)(\w)$", duration)
    if(not(m)):
        return(None)
    elif(m.group(2)=='d'): # days
        return(int(m.group(1))*60*24)
    if(m.group(2)=='h'): # hours
        return(int(m.group(1))*60)
    elif(m.group(2)=='s'): # seconds
        return(int(m.group(1))/60)
    else: # minutes
        return(int(m.group(1)))

def is_duration(s):
    '''
    check if string is duration
    '''

    # hour:minute
    if(':' in s):
        return(True)

    # 3h / 30m / etc.
    m = re.match("^(\d+)(\w)$", s)
    if(m):
        return(True)

    return(False)



import os

# where today directory will be
path = os.path.expanduser('~')
# today directory
path = os.path.join(path, '.today')

# theme directory
themes_path = os.path.join(path, 'themes')

# path where files are saved
save_path = os.path.join(path, 'saves')

# path to each files
data_path = os.path.join(path, "data.json")
settings_path = os.path.join(path, "settings.json")
purged_path = os.path.join(path, "purged.json")
yesterday_path = os.path.join(path, "yesterday.json")
notes_path = os.path.join(path, 'notes.json')
todo_path = os.path.join(path, 'todo.json')

# where the app is stored/installed
dir_path = os.path.dirname(__file__)

# source of themes
themes_source = os.path.join(dir_path, 'themes')

def check_and_create():
    '''
    create necessary directories if they do not exist
    '''

    # root directory
    if(not(os.path.exists(path))):
        print(f"creating directory: {path}")
        os.makedirs(path)

    # theme directory
    if(not(os.path.exists(themes_path))):
        os.makedirs(themes_path)

    # save directory (where saved datas will be)
    if(not(os.path.exists(save_path))):
        os.makedirs(save_path)



import json
import os
from shutil import copyfile

from . import paths
from . import settings_manupulation

# get settings data
settings = settings_manupulation.settings

def load():
    '''
    load theme
    '''

    update()
    with open(os.path.join(paths.themes_path, settings['theme'] + '.json'), 'r') as data:
            theme = json.load(data)
    return(theme)

def update():
    '''
    add themes from source that are not in themes directory
    '''

    # list of all the themes directory
    themes_ls = os.listdir(paths.themes_path)

    # for each theme not in themes directory, copy it there
    for theme in os.listdir(paths.themes_source):
        if(theme not in themes_ls):
            copyfile(os.path.join(paths.themes_source, theme), os.path.join(paths.themes_path, theme))
            print(f"Copied theme {theme} to {paths.themes_path}")

def create():
    '''
    create a new theme
    '''

    # name of theme file
    name = 'matrix'

    # theme
    theme = {
            'highlight': {
                'id': '\33[92m',
                'header': '\33[92m',
                'done': '\33[32m',
                'skip': '\33[30m',
                'undone': {
                    'even': '\33[40m\33[92m',
                    'odd': '\33[40m\33[92m'
                    },
                'next': {
                    'col': '\33[30m\33[102m',
                    'pointer': '\33[5m<---',
                },
            },
            'escape': '\33[0m',
        }

    # save the theme
    path = paths.themes_path + '/' + names + '.json'
    with open(path, 'w') as data:
        json.dump(theme, data)



#!/usr/bin/env python3
# import
import argparse
from sys import argv

from . import task_manupulation
from . import data_manupulation
from . import settings_manupulation
from . import todo
from . import time_formatting

def parse_arguments(args, a_id=None, a_name=None, a_duration=None, a_st=None, a_time_start=None, a_times=1, inc=0):
    '''
    argument parsing function
    '''
    init_id = a_id

    for i in range(a_times):
        # make ID = intial ID + increment value * (iteration number-1)
        if(type(a_id)==int):
            a_id = init_id + inc*i

        # task add
        if(args.a):
            task_manupulation.create(a_id, a_name, a_duration, start_at=a_st)

        # task done
        elif(args.d):
            if(args.E): # every
                task_manupulation.task_do_all()
            else:
                task_manupulation.task_do(a_id)

        # task undo
        elif(args.D):
            if(args.E): # every
                task_manupulation.task_undo_all()
            else:
                task_manupulation.task_undo(a_id)

        # task remove
        elif(args.r):
            task_manupulation.task_remove(a_id, every=args.E)

        # task skip
        elif(args.s):
            task_manupulation.task_skip(a_id, every=args.E)

        # task unskip
        elif(args.S):
            task_manupulation.task_unskip(a_id, every=args.E)

        # task modify
        elif(type(args.m)==int):
            task_manupulation.task_modify(args.m, a_id, a_name, a_duration, a_st, every=args.E)

        # data purge
        if(args.p):
            data_manupulation.purge()

        # data retrieve
        elif(args.P):
            data_manupulation.retrieve()

        # data newday
        elif(args.aa):
            data_manupulation.newday()

        # data yesterday
        elif(args.yy):
            task_manupulation.display_yesterday(a_id)

        # todo add
        if(args.A):
            todo.add(a_id, a_name, a_duration)

        # todo remove
        elif(args.R):
            todo.remove(a_id, every=args.E)

        # todo save
        elif(type(args.C)==int):
            task_manupulation.save_todo(args.C, a_id, every=args.E)

        # todo load 
        elif(type(args.L)==int):
            task_manupulation.load_todo(args.L, a_id, every=args.E)

        # todo load undone from yesterday
        elif(args.CY):
            task_manupulation.copy_todo_yesterday()

        # settings set day start
        elif(type(a_time_start)==int):
            settings_manupulation.change_time(a_time_start)

       # settings configure
        if(args.conf):
            settings_manupulation.change()

        # settings theme change
        elif(args.theme):
            settings_manupulation.change_theme()
        
        # data save file
        if(args.xs):
            data_manupulation.save(args.xs)

        # data load file
        elif(args.xl):
            data_manupulation.load(args.xl)

        # data delete file
        elif(args.xx):
            data_manupulation.delete(args.xx, every=args.E)

        # data list files
        elif(args.ls):
            data_manupulation.list()

        # task display
        if(len(argv)<=1 or args.y):
            task_manupulation.display_today(a_id)

    return(True)


# main
def main():
    '''
    main function
    '''

    # parse user arguments
    parser = argparse.ArgumentParser("today [Positoinal Arguments] [Optional Arguments]")

    ## argument groups
    args_task = parser.add_argument_group("Basic Task and Data")
    args_task_excl = args_task.add_mutually_exclusive_group()

    args_data = parser.add_argument_group("Advanced Data")
    args_data_excl = args_data.add_mutually_exclusive_group()

    args_extra = parser.add_argument_group("Extra")

    args_todo = parser.add_argument_group("Todo List")
    args_todo_excl = args_todo.add_mutually_exclusive_group()

    args_mod = parser.add_argument_group("Modifiers")
    args_mod_iteration_excl = args_mod.add_mutually_exclusive_group()

    args_settings = parser.add_argument_group("Settings")
    args_settings_excl = args_settings.add_mutually_exclusive_group()

    args_file = parser.add_argument_group("File")
    args_file_excl = args_file.add_mutually_exclusive_group()

    ## positional arguments
    parser.add_argument('arguments', metavar='ID Name Duration', nargs='*', help='[int] [str] [time]')

    ## task manupulation arguments
    args_task_excl.add_argument('-a', action='store_true', help='add/append Task [ID][Name][Duration]')
    args_task_excl.add_argument('-d', action='store_true', help='mark done [ID]')
    args_task_excl.add_argument('-D', action='store_true', help='mark undone [ID]')
    args_task_excl.add_argument('-r', action='store_true', help='remove [ID]')
    args_task_excl.add_argument('-s', action='store_true', help='skip [ID]')
    args_task_excl.add_argument('-S', action='store_true', help='unskip [ID]')
    args_task_excl.add_argument('-m', metavar='[ID]',  action='store', type=int, help='modify [New ID] [New Name] [New Duration]')
    args_task.add_argument('-y', action='store_true', help='display Tasks')

    ## advanced data manupulation arguments
    args_data_excl.add_argument('-p', action='store_true', help='purge Task Data')
    args_data_excl.add_argument('-P', action='store_true', help='retrieve purged Task Data')
    args_data_excl.add_argument('--aa', action='store_true', help='stat a new day')
    args_data_excl.add_argument('--yy', action='store_true', help='display yesterday')

    ## extra info arguments
    args_extra.add_argument('-t', metavar='[time]', action='store', type=str, help='Task start time')

    # todo list arguments
    args_todo_excl.add_argument('-A', action='store_true', help='add a new task to todo list [ID] [Name] [Duration]')
    args_todo_excl.add_argument('-R', action='store_true', help='remove a task from todo list [ID]')
    args_todo_excl.add_argument('-C', metavar='[ID]', action='store', type=int, help='copy a task to todo list')
    args_todo_excl.add_argument('-L', metavar='[ID]', action='store', type=int, help='load a task from todo list')
    args_todo_excl.add_argument('--CY', action='store_true', help='copy yesterday\'s undone tasks to todo list')

    ## modifier arguments
    args_mod.add_argument('-E', action='store_true', help='do action for every Tasks')
    args_mod_iteration_excl.add_argument('-i', metavar='[int]', action='store', type=int, help='iterate this number of times')
    args_mod_iteration_excl.add_argument('-I', metavar='[int]', action='store', type=int, help='iterate this number of times with incrementing ID')

    ## settings manupulation arguments
    args_settings_excl.add_argument('-T', metavar='[time]', action='store', type=str, help='day start time')
    args_settings_excl.add_argument('--conf', action='store_true', help='configure settings')
    args_settings_excl.add_argument('--theme', action='store_true', help='change theme')

    ## file management arguments
    args_file_excl.add_argument('--xs', metavar='[filename]', action='store', type=str, help='save')
    args_file_excl.add_argument('--xl', metavar='[filename]', action='store', type=str, help='load')
    args_file_excl.add_argument('--xx', metavar='[filename]', action='store', type=str, help='delete')
    args_file_excl.add_argument('--ls', action='store_true', help='list files')

    ##
    args = parser.parse_args()

    # number of times the action will be run and ID increation
    inc = 0
    a_times = 1
    if(args.i):
        a_times = args.i
    elif(args.I):
        a_times = args.I
        inc = 1

    # get ID, name and duration from user argument
    a_id, a_name, a_duration = None, None, None
    if(args.arguments):
        # if only one argument is passed (ID/Name/Duration)
        if(len(args.arguments) == 1): 
            if(args.arguments[0].isnumeric()):
                a_id = int(args.arguments[0])
            elif(time_formatting.is_duration(args.arguments[0])):
                a_duration = args.arguments[0]
            else:
                a_name = args.arguments[0]
        # first item is numeric (ID + Name and/or Duration)
        elif(args.arguments[0].isnumeric()):
            a_id = int(args.arguments[0])
            # last item is numeric/duration
            if(args.arguments[-1].isnumeric() or time_formatting.is_duration(args.arguments[-1])):
                a_duration = args.arguments[-1]
                # everything in the middle is Name
                if(len(args.arguments)>2):
                    a_name = " ".join(args.arguments[1:-1])
            # if not, everything to the end is Name
            else: 
                a_name = " ".join(args.arguments[1:])
        # (Name and/or Duration)
        else:
            # last item is numeric/duration
            if(args.arguments[-1].isnumeric() or time_formatting.is_duration(args.arguments[-1])):
                a_duration = args.arguments[-1]
                a_name = " ".join(args.arguments[:-1])
            # Name only
            else:
                a_name = " ".join(args.arguments)

    # turn duration into minutes
    if(a_duration):
        a_duration = time_formatting.to_min(a_duration)

    # turn start_at duration into minutes
    a_st = args.t 
    if(a_st):
        if(a_st == '-1'):
            a_st = -1
        else:
            a_st = time_formatting.to_min(a_st)

    # turn time start into minutes
    a_time_start = args.T
    if(a_time_start):
        a_time_start = time_formatting.to_min(a_time_start)

    parse_arguments(args=args, a_id=a_id, a_name=a_name, a_duration=a_duration, a_st=a_st, a_time_start=a_time_start, a_times=a_times, inc=inc)

    
if __name__ == "__main__":
    main()



import json
from os.path import join
from os import listdir, remove

from . import paths

tasks = {}

def newday():
    '''
    start a new task data
    '''

    global tasks

    # save task data as yesterday's
    with open(paths.yesterday_path, 'w') as data:
        json.dump(tasks, data)

    # make blank task data and save
    tasks = []
    write()

def purge():
    '''
    purge task data
    '''

    global tasks

    # sade task data as purged
    with open(paths.purged_path, 'w') as data:
        json.dump(tasks, data)

    # make blank task data and save
    tasks = []
    write()

def retrieve():
    '''
    retrieve purged data
    '''

    global tasks

    # open purged data and save as current task data
    try:
        with open(paths.purged_path, 'r') as data:
            tasks = json.load(data)
            write()

    except FileNotFoundError:
        print("Therer is no Purged Data file. Purged Data file is only generated when purge function is called.")

def write():
    '''
    write task dat
    '''

    with open(paths.data_path, 'w') as data:
        json.dump(tasks, data)

def read():
    '''
    read task data
    '''

    global tasks
    try:
        with open(paths.data_path, 'r') as data:
            tasks = json.load(data)
    except FileNotFoundError:
        print('Data file does not exist. Creating a new one.')
        tasks = []
        write()

def save(filename='default'):
    '''
    save task data in a file
    '''

    filename = filename + '.json'
    with open(join(paths.save_path, filename), 'w') as data:
        json.dump(tasks, data)

def load(filename='default'):
    '''
    load task data from a file
    '''

    filename = filename + '.json'
    global tasks
    try:
        with open(join(paths.save_path, filename), 'r') as data:
            save(filename='_') # backup current task data
            tasks = json.load(data)
            write()
    except FileNotFoundError:
        print(f"No file saved with the name {filename}.")

def delete(filename='default', every=False):
    '''
    delete a task file
    '''

    # delete everything
    if(every):
        for filename in listdir(paths.save_path):
            if(filename.endswith('.json')):
                file = join(paths.save_path, filename)
                remove(file)

    filename = filename + '.json'
    try:
        remove(join(paths.save_path, filename))
    except FileNotFoundError:
        print(f"No file saved with the name {filename}.")

def list():
    '''
    list all the task files
    '''

    for file in listdir(paths.save_path):
        if(file.endswith('.json')):
            print(file[:-5])

# read task data
read()



from setuptools import setup

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
        name='today_cli',
        version='1.0.0',
        description='Command Line Day Planner',
        long_description = long_description,
        long_description_content_type="text/markdown",
        url='https://github.com/TajwarHjkl/today/archive/refs/tags/v1.0.0.tar.gz',
        author='Takey Tajwar',
        author_email='tajwar.earth@proton.me',
        license='GPLv3',
        packages=['today'],
        scripts=['bin/today'],
        package_data={'today': ['themes/*.json']},
        zip_safe=False,
        classifiers=[
            "Programming Language :: Python :: 3",
            "Operating System :: OS Independent",
    ],
        )



import json
import os

from . import paths
from . import theme_manupulation
from . import theme_manupulation
from . import time_formatting

# the settings today will use if settings file does not exist or has missing attributes
default_settings = {
        'time_start': 420, # 7:00
        'theme': 'sunrise',
        'default': {
            'name': 'Unnamed',
            'duration': 30
            }
        }

# settings file
settings = {}

def write():
    '''
    write settings data
    '''

    with open(paths.settings_path, 'w') as data:
        json.dump(settings, data) 

def read():
    '''
    read settings data
    '''

    global settings

    # read settings file
    try:
        with open(paths.settings_path, 'r') as data:
            settings = json.load(data)
            update()

    # use default settings if settings file does not exist
    except FileNotFoundError:
        print('Settings file does not exist. Creating a new one.')
        settings = default_settings
        write()

def update():
    '''
    updates settings (add attributes that is unavailable in currents settings)
    '''

    global settings

    # check if each items in default settings exists in settings and add missing items
    for key, value in default_settings.items():
        updated = []

        if(not(key in settings)):
            settings[key] = value
            updated.append(key)

        if(updated):
            write()
            print(f"Settings were updated, new key(s) added: {updated}")

def change():
    '''
    change settings data
    '''

    global settings
    theme = theme_manupulation.load()

    def change_key(key, h, d=settings, func=None):
        '''
    # function for each keys to change
    '''

        # highlight name of the key
        print(f"{theme['highlight']['id']}{key}{theme['escape']}")

        # print help message
        print(h)

        # user input for new value
        new_value = input(f"Current: {d[key]}, New: ")

        # run a passed function
        if(func):
            new_value=func(new_value)

        # set new value
        if(new_value or type(new_value)==int): # user input != blank
            new_value = type(d[key])(new_value) # make data type of new value same as the old one
            d[key] = new_value

    print(f"{theme['highlight']['header']}today settings configurator{theme['escape']}")
    print("Write the new settings value for each key and press Enter; leave Blank and press Enter to keep the old value")

    # time start
    change_key('time_start', "When does your day start? By Default, it's 7 in the morning.", func=time_formatting.to_min)

    # default task data
    print(f"{theme['highlight']['undone']['even']}Default Task Values{theme['escape']}: the values today uses if user has not provided the argument when creating a new task")
    change_key('name', "default task name", d=settings['default'])
    change_key('duration', "default task duration", d=settings['default'])

    # write settings
    write()

def change_theme():
    '''
    change theme
    '''

    global settings

    # load theme
    theme = theme_manupulation.load()

    def theme_name(t):
        '''
    # theme name -= '.json'
    '''

        return(t[:-5])

    # show help
    print(f"Current theme: {settings['theme']}")
    print(f"{theme['highlight']['header']}Available Themes:{theme['escape']}")
    themes_ls = os.listdir(paths.themes_path)
    themes_ls = list(map(theme_name, themes_ls))
    for i, t in enumerate(themes_ls):
        print(f"{theme['highlight']['id']}{i}{theme['escape']}\t{t}")

    # user select new theme
    try:
        ui = int(input("Select theme: "))
        settings['theme'] = themes_ls[ui]

        # write settings
        write()

    except ValueError:
        print("Select using the ID number.")

    except IndexError:
        print("Invalid ID!")

def change_time(time):
    global settings

    settings['time_start'] = time

    write()

# check and create necessary files and directories
paths.check_and_create()

# read settings
read()
